/*.h2 Rich Text Format output driver (Acta)*/
/*.fo 1987 Maitreya Design */
/*
Writes Acta outline as RTF document

Written by David Dunham
*/
#include	<quickdraw.h>
#include	<toolutils.h>
#include	<memory.h>
#include	<dialogs.h>
#include	<errors.h>
#include	<events.h>
#include	<files.h>
#include	<packages.h>
#include	<printing.h>
#include	<resources.h>

typedef int				word;				/* Synonyms for VAX-hackers */
typedef unsigned char	byte;

#define TAB		'\t'
#define CR		'\r'

#define NIL		(0L)
#define TRUE	(-1)
#define FALSE	0

/* Label types */
#define	NONE	10
#define DECIMAL	11
#define ROMAN	12
#define BULLETS	14

/* Topic types */
#define TEXTtype	1
#define PICTtype	2

struct BLOCK_ATT {			/* Attribute portion of a topic */
	word	bType;			/* TEXTtype, PICTtype,  */
	word	bAttributes;	/* 0 */
	word	bFont;
	word	bStyle;
	word	bSize;
	byte	bColor;			/* Triangle color: index to COL# resource, 0 = black */
	byte	filler;			/* 0 */
	byte	firstLine;		/* Show first line only? 0xFF or 0 */
	byte	expanded;		/* Subtopics visible? 0xFF or 0 */
	word	hidden;			/* Is block hidden (by Collapse)? count of hide depth */
};
typedef struct BLOCK_ATT	BLOCK_ATT;

struct OUT_PARMS {
	word		fRef;				/* Data fork refNum */
	word		rRef;				/* Resource fork refNum */
	word		message;
	word		level;
	word		n;					/* Topics absolute number */
	char		*data;
	long		len;				/* Length of data */
	BLOCK_ATT	*att;
	THPrint		print_rec;
	word		already_error;
	char		*label;
	word		labelType;
	word		*raw_label;
	word		rulers;
	word		error_reporting;	/* Dp->dCtlRefNum or event */
	Handle		reserved;			/* Currently NIL */
	word		last_level;
	word		private;
};
typedef struct OUT_PARMS	*P_PTR;

/* Function prototypes */
void		file_error();
void		warp();
void		w_byte();
void		w_str();
OSErr		write_RTF();

/*
WRITE_RTF  Write Rich Text Format document
*/
OSErr write_RTF(p) register P_PTR p; {
	char			*s;
	long			count;
	OSErr			error;
	Handle			handle;
	register word	i;
	word			id, last_space, rsrc_fonts;
	Str255			name, string;
	WindowPtr		front;

	switch (p->message) {
	  case -1:
		w_str(p,"\P{\\rtf\\mac");
		/* Output font table (every installed font) */
		w_str(p,"\P{\\fonttbl");
		rsrc_fonts = CountResources('FONT');
		for (i = 1; i <= rsrc_fonts; i++) {
			SetResLoad(FALSE);			/* Dont actually load the fonts! */
			handle = GetIndResource('FONT',i);
			GetResInfo(handle,&id,&count,&name);
			SetResLoad(TRUE);			/* Back to normal */
			if (name.length != 0) {
				w_str(p,"\P\\f");
				NumToString((long)id >> 7,&string);
				w_str(p,&string);
				w_byte(p,' ');
				w_str(p,&name);
				w_byte(p,';');
			}
		}
		w_str(p,"\P}\r");
		break;
	  case 0:
		w_str(p,"\P{\\sl0\\f");
		NumToString((long)p->att->bFont,&string);
		w_str(p,&string);
		w_str(p,"\P\\fs");
		NumToString((long)p->att->bSize << 1,&string);
		w_str(p,&string);
		if (p->att->bStyle & bold) {
			w_str(p,"\P\\b");
		}
		if (p->att->bStyle & italic) {
			w_str(p,"\P\\i");
		}
		if (p->att->bStyle & underline) {
			w_str(p,"\P\\ulw");
		}
		if (p->att->bStyle & outline) {
			w_str(p,"\P\\outl");
		}
		/* Output ruler stuff */
		if (p->level > 1 && p->labelType == DECIMAL)
			i = 48;
		  else
			i = 32;
		if (p->labelType == BULLETS) i = 16;
		if (p->labelType == NONE) i = 0;
		/* Output first indent and left indent in twips (1/20 point) */
		w_str(p,"\P\\li");
		NumToString((long)(p->level * 16 + i) * 20,&string);
		w_str(p,&string);
		w_str(p,"\P\\fi");
		NumToString((long)i * -20,&string);
		w_str(p,&string);
		w_byte(p,' ');						/* Be sure not to run into text */
		/* Output label */
		if ((p->label)[0] != 0) {
			for (i = 1; i <= p->label[0]; i++)
				w_byte(p,p->label[i]);
		}
		if (p->att->bType == PICTtype) {
			w_str(p,"\P{\\pict\\macpict\\bin");
			NumToString((long)p->len,&string);
			w_str(p,&string);
			w_byte(p,' ');
			count = p->len;
			error = FSWrite(p->fRef,&count,p->data);
			w_byte(p,'}');
		  } else {							/* Not a PICT */
			warp(p,p->len,p->data);
		}
		w_str(p,"\P\\par}\r");
		break;
	  case 1:
		w_str(p,"\P}\r");
		break;
	}
	return(p->already_error);
}

/*
FILE_ERROR - Display the appropriate message for a file error
*/
void file_error(error,p) register OSErr error; P_PTR p; {

	if (p->already_error != noErr) return;	/* Only report once */
	p->already_error = error;				/* Indicate error */
}

/*
WARP  Escape {,\,},CR,TAB
*/
void warp(p,len,s) P_PTR p; long len; register char *s; {
	long			count;
	register word	i;
	register char	c;
	
	if (p->already_error) return;			/* Already encountered error */
	for (i = 0; i < len; i++) {
		c = *s++;
		switch (c) {
		  case '{':
		  case '}':
		  case '\\':
			w_byte(p,'\\'); w_byte(p,c);
			break;
		  case CR:
			w_str(p,"\P\\line ");
			break;
		  case TAB:
			w_str(p,"\P\\tab ");
			break;
		  default:
			w_byte(p,c);
		}
	}
}

/*
W_BYTE
*/
void w_byte(p,b) P_PTR p; unsigned char b; {
	long			count;
	register OSErr	error;
	
	if (p->already_error) return;			/* Already encountered error */
	count = sizeof(char);
	error = FSWrite(p->fRef,&count,&b);
	if (error) {
		file_error(error,p);
		return;
	}
}

/*
W_STR  Write a Pascal string
*/
void w_str(p,s) P_PTR p; char s[]; {
	long			count;
	register OSErr	error;

	if (p->already_error) return;			/* Already encountered error */
	count = s[0];
	error = FSWrite(p->fRef,&count,&s[1]);
	if (error) {
		file_error(error,p);
		return;
	}
}

/* Copyright 1987 Maitreya Design */
